@@ -1,6 +1,11 @@
 import unittest
 
-from urllib3.connectionpool import connection_from_url, HTTPConnectionPool
+from urllib3.connectionpool import (
+    connection_from_url,
+    HTTPConnection,
+    HTTPConnectionPool,
+)
+from urllib3.util import Timeout
 from urllib3.packages.ssl_match_hostname import CertificateError
 from urllib3.exceptions import (
     ClosedPoolError,
@@ -8,10 +13,10 @@ from urllib3.exceptions import (
     HostChangedError,
     MaxRetryError,
     SSLError,
-    TimeoutError,
+    ReadTimeoutError,
 )
 
-from socket import timeout as SocketTimeout
+from socket import error as SocketError, timeout as SocketTimeout
 from ssl import SSLError as BaseSSLError
 
 try:   # Python 3
@@ -23,6 +28,10 @@ except ImportError:
 
 
 class TestConnectionPool(unittest.TestCase):
+    """
+    Tests in this suite should exercise the ConnectionPool functionality
+    without actually making any network requests or connections.
+    """
     def test_same_host(self):
         same_host = [
             ('http://google.com/', '/'),
@@ -48,6 +57,7 @@ class TestConnectionPool(unittest.TestCase):
             c = connection_from_url(a)
             self.assertFalse(c.is_same_host(b), "%s =? %s" % (a, b))
 
+
     def test_max_connections(self):
         pool = HTTPConnectionPool(host='localhost', maxsize=1, block=True)
 
@@ -86,6 +96,25 @@ class TestConnectionPool(unittest.TestCase):
             str(EmptyPoolError(HTTPConnectionPool(host='localhost'), "Test.")),
             "HTTPConnectionPool(host='localhost', port=None): Test.")
 
+    def test_retry_exception_str(self):
+        self.assertEqual(
+            str(MaxRetryError(
+                HTTPConnectionPool(host='localhost'), "Test.", None)),
+            "HTTPConnectionPool(host='localhost', port=None): "
+            "Max retries exceeded with url: Test. (Caused by redirect)")
+
+        err = SocketError("Test")
+
+        # using err.__class__ here, as socket.error is an alias for OSError
+        # since Py3.3 and gets printed as this
+        self.assertEqual(
+            str(MaxRetryError(
+                HTTPConnectionPool(host='localhost'), "Test.", err)),
+            "HTTPConnectionPool(host='localhost', port=None): "
+            "Max retries exceeded with url: Test. "
+            "(Caused by {0}: Test)".format(str(err.__class__)))
+
+
     def test_pool_size(self):
         POOL_SIZE = 1
         pool = HTTPConnectionPool(host='localhost', maxsize=POOL_SIZE, block=True)
@@ -95,14 +124,13 @@ class TestConnectionPool(unittest.TestCase):
 
         def _test(exception, expect):
             pool._make_request = lambda *args, **kwargs: _raise(exception)
-            with self.assertRaises(expect):
-                pool.request('GET', '/')
+            self.assertRaises(expect, pool.request, 'GET', '/')
 
             self.assertEqual(pool.pool.qsize(), POOL_SIZE)
 
         #make sure that all of the exceptions return the connection to the pool
-        _test(Empty, TimeoutError)
-        _test(SocketTimeout, TimeoutError)
+        _test(Empty, ReadTimeoutError)
+        _test(SocketTimeout, ReadTimeoutError)
         _test(BaseSSLError, SSLError)
         _test(CertificateError, SSLError)
 
@@ -111,15 +139,15 @@ class TestConnectionPool(unittest.TestCase):
         # MaxRetryError, not EmptyPoolError
         # See: https://github.com/shazow/urllib3/issues/76
         pool._make_request = lambda *args, **kwargs: _raise(HTTPException)
-        with self.assertRaises(MaxRetryError):
-            pool.request('GET', '/', retries=1, pool_timeout=0.01)
+        self.assertRaises(MaxRetryError, pool.request,
+                          'GET', '/', retries=1, pool_timeout=0.01)
         self.assertEqual(pool.pool.qsize(), POOL_SIZE)
 
     def test_assert_same_host(self):
         c = connection_from_url('http://google.com:80')
 
-        with self.assertRaises(HostChangedError):
-            c.request('GET', 'http://yahoo.com:80', assert_same_host=True)
+        self.assertRaises(HostChangedError, c.request,
+                          'GET', 'http://yahoo.com:80', assert_same_host=True)
 
     def test_pool_close(self):
         pool = connection_from_url('http://google.com:80')
@@ -136,16 +164,28 @@ class TestConnectionPool(unittest.TestCase):
         pool.close()
         self.assertEqual(pool.pool, None)
 
-        with self.assertRaises(ClosedPoolError):
-            pool._get_conn()
+        self.assertRaises(ClosedPoolError, pool._get_conn)
 
         pool._put_conn(conn3)
 
-        with self.assertRaises(ClosedPoolError):
-            pool._get_conn()
+        self.assertRaises(ClosedPoolError, pool._get_conn)
 
-        with self.assertRaises(Empty):
-            old_pool_queue.get(block=False)
+        self.assertRaises(Empty, old_pool_queue.get, block=False)
+
+
+    def test_pool_timeouts(self):
+        pool = HTTPConnectionPool(host='localhost')
+        conn = pool._new_conn()
+        self.assertEqual(conn.__class__, HTTPConnection)
+        self.assertEqual(pool.timeout.__class__, Timeout)
+        self.assertEqual(pool.timeout._read, Timeout.DEFAULT_TIMEOUT)
+        self.assertEqual(pool.timeout._connect, Timeout.DEFAULT_TIMEOUT)
+        self.assertEqual(pool.timeout.total, None)
+
+        pool = HTTPConnectionPool(host='localhost', timeout=3)
+        self.assertEqual(pool.timeout._read, 3)
+        self.assertEqual(pool.timeout._connect, 3)
+        self.assertEqual(pool.timeout.total, None)
 
 
 if __name__ == '__main__':
